@@ -18,20 +18,9 @@
 
 """This module stores information and functionality that relates to plurals."""
 
-
-import gettext
-import locale
-import os
 import re
 import unicodedata
 
-
-try:
-    import pycountry
-except ImportError:
-    pycountry = None
-
-
 languages = {
     "ach": ("Acholi", 2, "n > 1"),
     "af": ("Afrikaans", 2, "(n != 1)"),
@@ -379,7 +368,7 @@ cldr_plural_categories = [
     "other",
 ]
 
-"""List of plural tags generated from CLDR 41.0.0 using
+"""List of plural tags generated from CLDR 44.0.1 using
 https://github.com/WeblateOrg/language-data
 """
 plural_tags = {
@@ -399,13 +388,14 @@ plural_tags = {
     "bez": ["one", "other"],
     "bg": ["one", "other"],
     "bho": ["one", "other"],
+    "blo": ["zero", "one", "other"],
     "bm": ["other"],
     "bn": ["one", "other"],
     "bo": ["other"],
     "br": ["one", "two", "few", "many", "other"],
     "brx": ["one", "other"],
     "bs": ["one", "few", "other"],
-    "ca": ["one", "other"],
+    "ca": ["one", "many", "other"],
     "ce": ["one", "other"],
     "ceb": ["one", "other"],
     "cgg": ["one", "other"],
@@ -443,7 +433,7 @@ plural_tags = {
     "gv": ["one", "two", "few", "other"],
     "ha": ["one", "other"],
     "haw": ["one", "other"],
-    "he": ["one", "two", "many", "other"],
+    "he": ["one", "two", "other"],
     "hi": ["one", "other"],
     "hnj": ["other"],
     "hr": ["one", "few", "other"],
@@ -488,6 +478,7 @@ plural_tags = {
     "lag": ["zero", "one", "other"],
     "lb": ["one", "other"],
     "lg": ["one", "other"],
+    "lij": ["one", "other"],
     "lkt": ["other"],
     "ln": ["one", "other"],
     "lo": ["other"],
@@ -502,7 +493,7 @@ plural_tags = {
     "mo": ["one", "few", "other"],
     "mr": ["one", "other"],
     "ms": ["other"],
-    "mt": ["one", "few", "many", "other"],
+    "mt": ["one", "two", "few", "many", "other"],
     "my": ["other"],
     "nah": ["one", "other"],
     "naq": ["one", "two", "other"],
@@ -587,6 +578,7 @@ plural_tags = {
     "ur": ["one", "other"],
     "uz": ["one", "other"],
     "ve": ["one", "other"],
+    "vec": ["one", "many", "other"],
     "vi": ["other"],
     "vo": ["one", "other"],
     "vun": ["one", "other"],
@@ -604,7 +596,8 @@ plural_tags = {
 
 
 def simplercode(code):
-    """This attempts to simplify the given language code by ignoring country
+    """
+    This attempts to simplify the given language code by ignoring country
     codes, for example.
 
     .. seealso::
@@ -619,8 +612,7 @@ def simplercode(code):
     separator = normalize_code(code).rfind("-")
     if separator >= 0:
         return code[:separator]
-    else:
-        return ""
+    return ""
 
 
 expansion_factors = {
@@ -637,145 +629,14 @@ langcode_ire = re.compile("^[a-z]{2,3}([_-][a-z]{2,3})?(@[a-z0-9]+)?$", re.IGNOR
 variant_re = re.compile("^[_-][A-Z]{2,3}(@[a-zA-Z0-9]+|)$")
 
 
-def languagematch(languagecode, otherlanguagecode):
-    """matches a languagecode to another, ignoring regions in the second"""
-    if languagecode is None:
-        return langcode_re.match(otherlanguagecode)
-    return languagecode == otherlanguagecode or (
-        otherlanguagecode.startswith(languagecode)
-        and variant_re.match(otherlanguagecode[len(languagecode) :])
-    )
-
-
-def get_country_iso_name(country_code):
-    """Return country ISO name."""
-    country_code = country_code.upper()
-    try:
-        if len(country_code) == 2:
-            country = pycountry.countries.get(alpha_2=country_code)
-        else:
-            country = pycountry.countries.get(alpha_3=country_code)
-        if hasattr(country, "common_name"):
-            return country.common_name
-        return country.name
-    except (KeyError, AttributeError):
-        return ""
-
-
-def get_language_iso_name(language_code):
-    """Return language ISO name."""
-    try:
-        if len(language_code) == 2:
-            language = pycountry.languages.get(alpha_2=language_code)
-        else:
-            language = pycountry.languages.get(alpha_3=language_code)
-        if hasattr(language, "common_name"):
-            return language.common_name
-        return language.name
-    except (KeyError, AttributeError):
-        return ""
-
-
-def get_language_iso_fullname(language_code):
-    """Return language ISO fullname.
-
-    If language code is not a simple ISO 639 code, then we try to split into a
-    two part language code (ISO 639 and ISO 3166).
-    """
-    if len(language_code) > 3:
-        language_code = language_code.replace("_", "-").replace("@", "-")
-        language_code = "-".join(language_code.split("-")[:2])
-        if "-" not in language_code:
-            return ""
-        language_code, country_code = language_code.split("-")
-        language_name = get_language_iso_name(language_code)
-        if not language_name:
-            return ""
-        country_name = get_country_iso_name(country_code)
-        if not country_name:
-            return ""
-        return f"{language_name} ({country_name})"
-    return get_language_iso_name(language_code)
-
-
 dialect_name_re = re.compile(r"(.+)\s\(([^)\d]{,25})\)$")
 # The limit of 25 characters on the country name is so that "Interlingua (...)"
 # (see above) is correctly interpreted.
 
 
-def tr_lang(langcode=None):
-    """Gives a function that can translate a language name, even in the form
-    ``"language (country)"``, into the language with iso code langcode, or the
-    system language if no language is specified.
-    """
-    langfunc = gettext_lang(langcode)
-    countryfunc = gettext_country(langcode)
-
-    def handlelanguage(name):
-        match = dialect_name_re.match(name)
-        if match:
-            language, country = match.groups()
-            if country != "macrolanguage":
-                return "{} ({})".format(
-                    _fix_language_name(langfunc(language)),
-                    countryfunc(country),
-                )
-        return _fix_language_name(langfunc(name))
-
-    return handlelanguage
-
-
-def _fix_language_name(name):
-    """Identify and replace some unsightly names present in iso-codes.
-
-    If the name is present in _fixed_names we assume it is untranslated and we
-    replace it with a more usable rendering.  If the remaining part is long and
-    includes a semi-colon, we only take the text up to the semi-colon to keep
-    things neat.
-    """
-    if name in _fixed_names:
-        return _fixed_names[name]
-    elif len(name) > 11:
-        # These constants are somewhat arbitrary, but testing with the Japanese
-        # translation of ISO codes suggests these as the upper bounds.
-        split_point = name[5:].find(";")
-        if split_point >= 0:
-            return name[: 5 + split_point]
-    return name
-
-
-def gettext_domain(langcode, domain, localedir=None):
-    """Returns a gettext function for given iso domain"""
-    kwargs = dict(domain=domain, localedir=localedir, fallback=True)
-    if langcode:
-        kwargs["languages"] = [langcode]
-    elif os.name == "nt":
-        # On Windows the default locale is not used for some reason
-        kwargs["languages"] = [locale.getdefaultlocale()[0]]
-    t = gettext.translation(**kwargs)
-    return t.gettext
-
-
-def gettext_lang(langcode=None):
-    """Returns a gettext function to translate language names into the given
-    language, or the system language if no language is specified.
-    """
-    if pycountry is None:
-        return gettext_domain(langcode, "iso_639")
-    return gettext_domain(langcode, "iso639-3", pycountry.LOCALES_DIR)
-
-
-def gettext_country(langcode=None):
-    """Returns a gettext function to translate country names into the given
-    language, or the system language if no language is specified.
-    """
-    if pycountry is None:
-        return gettext_domain(langcode, "iso_3166")
-    return gettext_domain(langcode, "iso3166", pycountry.LOCALES_DIR)
-
-
 def normalize(string, normal_form="NFC"):
-    """Return a unicode string in its normalized form
+    """
+    Return a unicode string in its normalized form.
 
     :param string: The string to be normalized
     :param normal_form: NFC (default), NFD, NFKC, NFKD
@@ -796,9 +657,10 @@ __normalised_languages = {normalize_code(key) for key in languages}
 
 
 def simplify_to_common(language_code):
-    """Simplify language code to the most commonly used form for the language,
+    """
+    Simplify language code to the most commonly used form for the language,
     stripping country information for languages that tend not to be localized
-    differently for different countries
+    differently for different countries.
     """
     simpler = simplercode(language_code)
     if simpler == "":
