@@ -16,20 +16,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""Module for handling XLIFF files for translation.
+"""
+Module for handling XLIFF files for translation.
 
 The official recommendation is to use the extention .xlf for XLIFF files.
 """
 
+import contextlib
+
 from lxml import etree
 
 from translate.misc.multistring import multistring
-from translate.misc.xml_helpers import getXMLspace, setXMLlang, setXMLspace
+from translate.misc.xml_helpers import (
+    clear_content,
+    getXMLspace,
+    setXMLlang,
+    setXMLspace,
+)
 from translate.storage import base, lisa
 from translate.storage.placeables.lisa import strelem_to_xml, xml_to_strelem
 from translate.storage.workflow import StateEnum as state
 
-
 # TODO: handle translation types
 
 ID_SEPARATOR = "\04"
@@ -140,7 +147,6 @@ class xliffunit(lisa.LISAunit):
 
     def createlanguageNode(self, lang, text, purpose):
         """Returns an xml Element setup with given parameters."""
-
         # TODO: for now we do source, but we have to test if it is target,
         # perhaps with parameter. Alternatively, we can use lang, if
         # supplied, since an xliff file has to conform to the bilingual
@@ -182,9 +188,7 @@ class xliffunit(lisa.LISAunit):
             self.set_source_dom(sourcelanguageNode)
 
         # Clear sourcelanguageNode first
-        for i in range(len(sourcelanguageNode)):
-            del sourcelanguageNode[0]
-        sourcelanguageNode.text = None
+        clear_content(sourcelanguageNode)
 
         strelem_to_xml(sourcelanguageNode, value[0])
 
@@ -216,16 +220,15 @@ class xliffunit(lisa.LISAunit):
             self.set_target_dom(languageNode, append)
 
         # Clear languageNode first
-        for i in range(len(languageNode)):
-            del languageNode[0]
-        languageNode.text = None
+        clear_content(languageNode)
 
         strelem_to_xml(languageNode, value[0])
         ### currently giving some issues in Virtaal: self._rich_target = value
 
     def get_rich_target(self, lang=None):
-        """retrieves the "target" text (second entry), or the entry in the
-        specified language, if it exists
+        """
+        retrieves the "target" text (second entry), or the entry in the
+        specified language, if it exists.
         """
         if self._rich_target is None:
             self._rich_target = [
@@ -247,12 +250,12 @@ class xliffunit(lisa.LISAunit):
     def addalttrans(
         self, txt, origin=None, lang=None, sourcetxt=None, matchquality=None
     ):
-        """Adds an alt-trans tag and alt-trans components to the unit.
+        """
+        Adds an alt-trans tag and alt-trans components to the unit.
 
         :type txt: String
         :param txt: Alternative translation of the source text.
         """
-
         # TODO: support adding a source tag ad match quality attribute.  At the
         # source tag is needed to inject fuzzy matches from a TM.
         alttrans = etree.SubElement(self.xmlelement, self.namespaced("alt-trans"))
@@ -270,7 +273,8 @@ class xliffunit(lisa.LISAunit):
             setXMLlang(alttrans, lang)
 
     def getalttrans(self, origin=None):
-        """Returns <alt-trans> for the given origin as a list of units. No
+        """
+        Returns <alt-trans> for the given origin as a list of units. No
         origin means all alternatives.
         """
         translist = []
@@ -282,12 +286,10 @@ class xliffunit(lisa.LISAunit):
 
                 # the source tag is optional
                 sourcenode = node.iterdescendants(self.namespaced("source"))
-                try:
+                with contextlib.suppress(StopIteration):
                     newunit.source = lisa.getText(
                         next(sourcenode), getXMLspace(node, self._default_xml_space)
                     )
-                except StopIteration:
-                    pass
 
                 # must have one or more targets
                 targetnode = node.iterdescendants(self.namespaced("target"))
@@ -302,11 +304,11 @@ class xliffunit(lisa.LISAunit):
         return translist
 
     def delalttrans(self, alternative):
-        """Removes the supplied alternative from the list of alt-trans tags"""
+        """Removes the supplied alternative from the list of alt-trans tags."""
         self.xmlelement.remove(alternative.xmlelement)
 
     def addnote(self, text, origin=None, position="append"):
-        """Add a note specifically in a "note" tag"""
+        """Add a note specifically in a "note" tag."""
         if position != "append":
             self.removenotes(origin=origin)
 
@@ -320,7 +322,8 @@ class xliffunit(lisa.LISAunit):
             note.set("from", origin)
 
     def _getnotelist(self, origin=None):
-        """Returns the text from notes matching ``origin`` or all notes.
+        """
+        Returns the text from notes matching ``origin`` or all notes.
 
         :param origin: The origin of the note (or note type)
         :type origin: String
@@ -378,8 +381,7 @@ class xliffunit(lisa.LISAunit):
         if targetnode is None:
             if self.isapproved():
                 return self.S_UNREVIEWED
-            else:
-                return self.S_UNTRANSLATED
+            return self.S_UNTRANSLATED
 
         xmlstate = targetnode.get("state", None)
         state_n = self.statemap.get(xmlstate, self.S_UNTRANSLATED)
@@ -405,10 +407,9 @@ class xliffunit(lisa.LISAunit):
         if value == self.S_UNTRANSLATED:
             if targetnode is not None and "state" in targetnode.attrib:
                 del targetnode.attrib["state"]
-        else:
-            if targetnode is not None:
-                xmlstate = self.statemap_r.get(value)
-                targetnode.set("state", xmlstate)
+        elif targetnode is not None:
+            xmlstate = self.statemap_r.get(value)
+            targetnode.set("state", xmlstate)
 
         self.markapproved(value > self.S_NEEDS_REVIEW)
 
@@ -424,11 +425,12 @@ class xliffunit(lisa.LISAunit):
             self.xmlelement.set("approved", "no")
 
     def isreview(self):
-        """States whether this unit needs to be reviewed"""
+        """States whether this unit needs to be reviewed."""
         return self.get_state_id() == self.S_NEEDS_REVIEW
 
     def markreviewneeded(self, needsreview=True, explanation=None):
-        """Marks the unit to indicate whether it needs review.
+        """
+        Marks the unit to indicate whether it needs review.
 
         Adds an optional explanation as a note.
         """
@@ -506,19 +508,29 @@ class xliffunit(lisa.LISAunit):
         return uid
 
     def addlocation(self, location):
-        self.setid(location)
+        if ":" in location:
+            sourcefile, linenumber = location.rsplit(":", 1)
+            contexts = [("sourcefile", sourcefile), ("linenumber", linenumber)]
+        else:
+            contexts = [("sourcefile", location)]
+        self.createcontextgroup("", contexts, "location")
 
     def getlocations(self):
-        id_attr = str(self.xmlelement.get("id") or "")
-        # XLIFF files downloaded from PO projects in Pootle
-        # might have id equal to .source, so let's avoid
-        # that:
-        if id_attr and id_attr != self.source:
-            return [id_attr]
-        return []
+        """Returns a list of locations."""
+        locations = []
+        for contextgroup in self.getcontextgroupsbyattribute("purpose", "location"):
+            sourcefile = next((x for x in contextgroup if x[0] == "sourcefile"), None)
+            linenumber = next((x for x in contextgroup if x[0] == "linenumber"), None)
+            if sourcefile is not None and linenumber is not None:
+                locations.append(sourcefile[1] + ":" + linenumber[1])
+            elif sourcefile is not None:
+                locations.append(sourcefile[1])
+
+        return locations
 
     def createcontextgroup(self, name, contexts=None, purpose=None):
-        """Add the context group to the trans-unit with contexts a list with
+        """
+        Add the context group to the trans-unit with contexts a list with
         (type, text) tuples describing each context.
         """
         assert contexts
@@ -539,29 +551,48 @@ class xliffunit(lisa.LISAunit):
             context.set("context-type", type)
 
     def getcontextgroups(self, name):
-        """Returns the contexts in the context groups with the specified name"""
+        """Returns the contexts in the context groups with the specified name."""
         groups = []
         grouptags = self.xmlelement.iterdescendants(self.namespaced("context-group"))
         # TODO: conbine name in query
         for group in grouptags:
             if group.get("name") == name:
                 contexts = group.iterdescendants(self.namespaced("context"))
-                pairs = []
-                for context in contexts:
-                    pairs.append(
-                        (
-                            context.get("context-type"),
-                            lisa.getText(
-                                context,
-                                getXMLspace(self.xmlelement, self._default_xml_space),
-                            ),
-                        )
+                pairs = [
+                    (
+                        context.get("context-type"),
+                        lisa.getText(
+                            context,
+                            getXMLspace(self.xmlelement, self._default_xml_space),
+                        ),
                     )
+                    for context in contexts
+                ]
+                groups.append(pairs)  # not extend
+        return groups
+
+    def getcontextgroupsbyattribute(self, attributeName, attributeValue):
+        """Returns the contexts in the context groups with the specified attributeName and attributeValue."""
+        groups = []
+        grouptags = self.xmlelement.iterdescendants(self.namespaced("context-group"))
+        for group in grouptags:
+            if group.get(attributeName) == attributeValue:
+                contexts = group.iterdescendants(self.namespaced("context"))
+                pairs = [
+                    (
+                        context.get("context-type"),
+                        lisa.getText(
+                            context,
+                            getXMLspace(self.xmlelement, self._default_xml_space),
+                        ),
+                    )
+                    for context in contexts
+                ]
                 groups.append(pairs)  # not extend
         return groups
 
     def getrestype(self):
-        """returns the restype attribute in the trans-unit tag"""
+        """Returns the restype attribute in the trans-unit tag."""
         return self.xmlelement.get("restype")
 
     def merge(self, otherunit, overwrite=False, comments=True, authoritative=False):
@@ -578,20 +609,19 @@ class xliffunit(lisa.LISAunit):
         if comments:
             self.addnote(otherunit.getnotes())
 
-    def correctorigin(self, node, origin):
-        """Check against node tag's origin (e.g note or alt-trans)"""
-        if origin is None:
-            return True
-        elif origin in node.get("from", ""):
-            return True
-        elif origin in node.get("origin", ""):
-            return True
-        else:
-            return False
+    @staticmethod
+    def correctorigin(node, origin):
+        """Check against node tag's origin (e.g note or alt-trans)."""
+        return (
+            origin is None
+            or origin in node.get("from", "")
+            or origin in node.get("origin", "")
+        )
 
     @classmethod
     def multistring_to_rich(cls, mstr):
-        """Override :meth:`TranslationUnit.multistring_to_rich` which is used
+        """
+        Override :meth:`TranslationUnit.multistring_to_rich` which is used
         by the ``rich_source`` and ``rich_target`` properties.
         """
         strings = mstr
@@ -604,7 +634,8 @@ class xliffunit(lisa.LISAunit):
 
     @classmethod
     def rich_to_multistring(cls, elem_list):
-        """Override :meth:`TranslationUnit.rich_to_multistring` which is used
+        """
+        Override :meth:`TranslationUnit.rich_to_multistring` which is used
         by the ``rich_source`` and ``rich_target`` properties.
         """
         return multistring([str(elem) for elem in elem_list])
@@ -639,7 +670,7 @@ class xlifffile(lisa.LISAfile):
 
     def initbody(self):
         # detect the xliff namespace, handle both 1.1 and 1.2
-        for prefix, ns in self.document.getroot().nsmap.items():
+        for ns in self.document.getroot().nsmap.values():
             if ns and ns.startswith(self.unversioned_namespace):
                 self.namespace = ns
                 break
@@ -658,12 +689,12 @@ class xlifffile(lisa.LISAfile):
 
     def addheader(self):
         """Initialise the file header."""
-        pass
 
     def createfilenode(
         self, filename, sourcelanguage=None, targetlanguage=None, datatype="plaintext"
     ):
-        """creates a filenode with the given filename. All parameters are
+        """
+        creates a filenode with the given filename. All parameters are
         needed for XLIFF compliance.
         """
         if sourcelanguage is None:
@@ -689,16 +720,18 @@ class xlifffile(lisa.LISAfile):
         etree.SubElement(filenode, self.namespaced(self.bodyNode))
         return filenode
 
-    def getfilename(self, filenode):
-        """returns the name of the given file"""
+    @staticmethod
+    def getfilename(filenode):
+        """Returns the name of the given file."""
         return filenode.get("original")
 
-    def setfilename(self, filenode, filename):
-        """set the name of the given file"""
+    @staticmethod
+    def setfilename(filenode, filename):
+        """Set the name of the given file."""
         return filenode.set("original", filename)
 
     def getfilenames(self):
-        """returns all filenames in this XLIFF file"""
+        """Returns all filenames in this XLIFF file."""
         filenodes = self.document.getroot().iterchildren(self.namespaced("file"))
         filenames = [self.getfilename(filenode) for filenode in filenodes]
         filenames = list(filter(None, filenames))
@@ -707,7 +740,7 @@ class xlifffile(lisa.LISAfile):
         return filenames
 
     def getfilenode(self, filename, createifmissing=False):
-        """finds the filenode with the given name"""
+        """Finds the filenode with the given name."""
         filenodes = self.document.getroot().iterchildren(self.namespaced("file"))
         for filenode in filenodes:
             if self.getfilename(filenode) == filename:
@@ -716,17 +749,6 @@ class xlifffile(lisa.LISAfile):
             return self.createfilenode(filename)
         return None
 
-    def getids(self, filename=None):
-        if not filename:
-            return super().getids()
-
-        self.id_index = {}
-        prefix = filename + ID_SEPARATOR
-        units = (unit for unit in self.units if unit.getid().startswith(prefix))
-        for index, unit in enumerate(units):
-            self.id_index[unit.getid()[len(prefix) :]] = unit
-        return self.id_index.keys()
-
     def setsourcelanguage(self, language):
         if not language:
             return
@@ -752,7 +774,8 @@ class xlifffile(lisa.LISAfile):
     targetlanguage = property(gettargetlanguage, settargetlanguage)
 
     def getdatatype(self, filename=None):
-        """Returns the datatype of the stored file. If no filename is given,
+        """
+        Returns the datatype of the stored file. If no filename is given,
         the datatype of the first file is given.
         """
         if filename:
@@ -766,7 +789,8 @@ class xlifffile(lisa.LISAfile):
         return ""
 
     def getdate(self, filename=None):
-        """Returns the date attribute for the file.
+        """
+        Returns the date attribute for the file.
 
         If no filename is given, the date of the first file is given.
         If the date attribute is not specified, None is returned.
@@ -785,7 +809,8 @@ class xlifffile(lisa.LISAfile):
         return None
 
     def removedefaultfile(self):
-        """We want to remove the default file-tag as soon as possible if we
+        """
+        We want to remove the default file-tag as soon as possible if we
         know if still present and empty.
         """
         filenodes = list(self.document.getroot().iterchildren(self.namespaced("file")))
@@ -798,7 +823,7 @@ class xlifffile(lisa.LISAfile):
                 break
 
     def getheadernode(self, filenode, createifmissing=False):
-        """finds the header node for the given filenode"""
+        """Finds the header node for the given filenode."""
         # TODO: Deprecated?
         headernode = filenode.iterchildren(self.namespaced("header"))
         try:
@@ -807,11 +832,10 @@ class xlifffile(lisa.LISAfile):
             pass
         if not createifmissing:
             return None
-        headernode = etree.SubElement(filenode, self.namespaced("header"))
-        return headernode
+        return etree.SubElement(filenode, self.namespaced("header"))
 
     def getbodynode(self, filenode, createifmissing=False):
-        """finds the body node for the given filenode"""
+        """Finds the body node for the given filenode."""
         bodynode = filenode.iterchildren(self.namespaced("body"))
         try:
             return next(bodynode)
@@ -830,9 +854,10 @@ class xlifffile(lisa.LISAfile):
         super().addunit(unit, new=new)
 
     def addsourceunit(self, source, filename="NoName", createifmissing=False):
-        """adds the given trans-unit to the last used body node if the filename
+        """
+        adds the given trans-unit to the last used body node if the filename
         has changed it uses the slow method instead (will create the nodes
-        required if asked). Returns success
+        required if asked). Returns success.
         """
         if not self.switchfile(filename, createifmissing):
             return None
@@ -845,7 +870,8 @@ class xlifffile(lisa.LISAfile):
         return unit
 
     def switchfile(self, filename, createifmissing=False):
-        """Adds the given trans-unit (will create the nodes required if asked).
+        """
+        Adds the given trans-unit (will create the nodes required if asked).
 
         :returns: Success
         :rtype: Boolean
@@ -864,7 +890,7 @@ class xlifffile(lisa.LISAfile):
         return self.body is not None
 
     def creategroup(self, filename="NoName", createifmissing=False, restype=None):
-        """adds a group tag into the specified file"""
+        """Adds a group tag into the specified file."""
         if not self.switchfile(filename, createifmissing):
             return None
         group = etree.SubElement(self.body, self.namespaced("group"))
@@ -878,7 +904,7 @@ class xlifffile(lisa.LISAfile):
 
     @classmethod
     def parsestring(cls, storestring):
-        """Parses the string to return the correct file object"""
+        """Parses the string to return the correct file object."""
         xliff = super().parsestring(storestring)
         if xliff.units:
             header = xliff.units[0]
